@@ -185,14 +185,18 @@ zfsdev_close(void *data)
 	zfsdev_state_t *zs = data;
 
 	ASSERT(zs != NULL);
-	ASSERT3S(zs->zs_minor, >, 0);
 
+	mutex_enter(&zfsdev_state_lock);
+
+	ASSERT(zs->zs_minor != 0);
+
+	zs->zs_minor = -1;
 	zfs_onexit_destroy(zs->zs_onexit);
 	zfs_zevent_destroy(zs->zs_zevent);
 	zs->zs_onexit = NULL;
 	zs->zs_zevent = NULL;
-	membar_producer();
-	zs->zs_minor = -1;
+
+	mutex_exit(&zfsdev_state_lock);
 }
 
 static int
@@ -367,3 +371,4 @@ MODULE_DEPEND(zfsctrl, krpc, 1, 1, 1);
 #endif
 MODULE_DEPEND(zfsctrl, acl_nfs4, 1, 1, 1);
 MODULE_DEPEND(zfsctrl, crypto, 1, 1, 1);
+MODULE_DEPEND(zfsctrl, zlib, 1, 1, 1);
